/*
 * 2005/09/05 - Len Meakin.
 *
 * relay.c: simple program that performs the following:
 *
 * A: Sends message to task B
 * B: Receives message then Relays it to task C
 * C: Receives message (as if it has come from task A) then Replys to task A.
 *
 */

/*--------------------------------------------------------------------------*\
 * System includes.
\*--------------------------------------------------------------------------*/
#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/*--------------------------------------------------------------------------*\
 * SRR include.
\*--------------------------------------------------------------------------*/
#include "srr.h"

/*--------------------------------------------------------------------------*\
 * Main entry point.
\*--------------------------------------------------------------------------*/
int 
main									(int			 argc, 
										 char			*argv[])
{
	pid_t	client 		 = -1;
	pid_t	relay		 = -1;
	pid_t	recv		 = -1;
	int		opt			 =  0;
	int		loop		 =  0;
	char	smsg[BUFSIZ] = {0};
	char	rmsg[BUFSIZ] = {0};

	/*-- Parse command line arguments ---------------------------------*/
	while ((opt = getopt (argc, argv, "l:")) != -1)
	{	switch(opt)
		{ case 'l': loop = atoi(optarg); break;
		  default :	exit(1);
		}
	}


	/*-- Regiseter with SRR module ------------------------------------*/
	if (SrrReg() == -1)
	{	
		printf("relay [SEND] [%d]: InitSrr() failed: [%d] %s\r\n",
				getpid(), errno, strerror(errno));
		exit(1);
	}


	/*-- Fork our Relay receive handler -------------------------------*/
	if ((relay = fork()) == -1)
	{	
		printf ("relay [RELAY][%d]: fork failed: [%d] %s\r\n",
				 getpid(), errno, strerror(errno));
		exit (1);

	}else if (relay == 0)
	{
		do
		{	/*-- Wait in receive state --------------------------------*/
			if ((client = Receive (0, rmsg, BUFSIZ)) == -1)
			{	
				printf ("relay [RELAY][%d]: Receive failed: [%d] %s\r\n",
						 getpid(), errno, strerror(errno));
				exit(1);
			}
			
			/*-- Successfully received relayed message ----------------*/
			printf("relay [RELAY][%d]: received message: \"%s\"\r\n", getpid(), rmsg);
			
			/*-- Replay back to the client (original sender) ----------*/
			printf("relay [RELAY][%d]: sending reply: \"%s\"\r\n", getpid(), "OK");
			Reply (client, "OK", 3);
		
		} while (--loop > 0);
		return 0;

	}else
	{
		/*-- Fork our Receive hanlder ---------------------------------*/
		if ((recv = fork()) == -1)
		{	
			printf ("relay [RECV] [%d]: fork failed: [%d] %s\r\n",
				getpid(), errno, strerror(errno));
			exit (1);

		}else if (recv == 0)
		{
			do
			{	/*-- Wait in receive state ----------------------------*/
				if((client = Receive (0, rmsg, BUFSIZ)) == -1)
				{
					printf ("relay [RECV] [%d]: Receive failed: [%d] %s\n",
							getpid(), errno, strerror(errno));
					exit(1);
				}
				printf("relay [RECV] [%d]: received message: \"%s\"\n", getpid(), rmsg);
				printf("relay [RECV] [%d]: relaying message to %d\n", getpid(), relay);
				
				/*-- Relay message to our forked child ----------------*/
				if (Relay (client, relay) == -1)
				{
					printf ("relay [RECV] [%d]: Relay failed: [%d] %s\n",
							getpid(), errno, strerror(errno));
							
					/*-- Replay with error to client ------------------*/
					printf("relay [RELAY][%d]: sending reply: \"%s\"\r\n", getpid(), "NOK");
					Reply (client, "NOK", 4);
					
					/*-- Termineate relay process ---------------------*/
					kill (relay, SIGKILL);
					exit(1);
				}
				
			}while (--loop > 0);
					
		}else
		{
			/*-- Buildup send message ---------------------------------*/
			sprintf(smsg, "Relay Test Message [%d]", getpid());
			
			do
			{	/*-- Log our process identifier -----------------------*/
				printf ("relay [SEND] [%d]: send message: \"%s\"\r\n", getpid(), smsg);

				/*-- Send message to our receive handler --------------*/
				if ((Send (recv, smsg, rmsg, BUFSIZ, BUFSIZ)) == -1)
				{	
					printf ("relay [SEND] [%d]: Send failed: [%d] %s\n",
							getpid(), errno, strerror(errno));
					exit(1);
					
				}else
				{
					printf ("relay [SEND] [%d]: reply [%s]\n", getpid(), rmsg);
				}
				
			} while (--loop > 0);
		}		
	}
	return 0;
}

