/*
** rwmtest.c - Test message passing using Writemsg() / Radmsg()
**             on the same node and through the network
** 
** Usage: rwmsg [[-s|-r] name]
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/kernel.h>

void dumpbuf(char *buf, int len)
{
	int i;
	for (i = 0; i < len; ++i)
		if (isprint(buf[i]))
			putchar(buf[i]);
		else
			putchar('?');
}

int sender(pid_t pid)
{
	char rbuf[16], sbuf[16];
	int rlen, slen, n = 2;

	strcpy(sbuf,"abcdefghijkl");
	slen = strlen(sbuf);

	while(n--)
	{
		strcpy(rbuf,"0123456789");
		rlen = strlen(rbuf);

		sleep(1);
		printf("Sender   %d:              send buf = %s\n",getpid(), sbuf);
		printf("Sender   %d:      before-reply buf = %s\n",getpid(), rbuf);
		if (Send(pid,sbuf,rbuf,slen,rlen) == -1)
			return -1;
		printf("Sender   %d:       after-reply buf = ",getpid());

		dumpbuf(rbuf, rlen);
		puts("");

		sleep(1);
		puts("");
	}

	return 0;
}

int receiver(pid_t pid)
{
	int wlen,rlen;
	char rmbuf[32];
	char *lwmbuf = "ABCDEFGHIJKL";
	char *swmbuf = "+++";

	/*
	** Here I write at an offset into the reply buffer and don't write
	** anything with Reply(): if sender and receiver are on the same
	** node, the result is what we expect (only the part of the reply buffer
	** overwritten by Writemsg() is modified, the rest is intact), but
	** if they are on different nodes, the reply buffer contains part of
	** the send buffer: i.e. even QNX4 uses a single buffer for message
	** passing when the receiver can't direclty access the sender's memory!
	*/
	printf("Receiver %d: Test partial Writemsg()\n",getpid());
	if ((pid = Receive(pid,0,0)) == -1)
		return -1;

	printf("Receiver %d: write offset = %d, buf =     %s\n",getpid(), 4, swmbuf);
	wlen = Writemsg(pid, 4, swmbuf, strlen(swmbuf));

	Reply(pid,0,0);
	sleep(1);
	puts("");

	/*
	** Here I mix Readmsg() and Writemsg(): if sender and receiver are on
	** the same node, the result is what we expect (the first Writemsg() doesn't
	** interfere with the following Readmsg()), but if they are on different
	** nodes not only Readmsg() reads what Writemsg() has written before,
	** but it also reads past the sender's buffer: again, a single buffer
	** is used when message passing happens through the network.
	*/
	printf("Receiver %d: Test mixed Readmsg()/Writemsg()\n",getpid());
	if ((pid = Receive(pid,0,0)) == -1)
		return -1;

	printf("Receiver %d:  read offset = %d, buf = ",getpid(), 0);
	rlen = Readmsg(pid, 0, rmbuf, sizeof(rmbuf));
	dumpbuf(rmbuf, rlen);
	puts("");

	printf("Receiver %d: write offset = %d, buf =     %s\n",getpid(), 4, swmbuf);
	wlen = Writemsg(pid, 4, swmbuf, strlen(swmbuf));
	
	printf("Receiver %d:  read offset = %d, buf =   ",getpid(), 2);
	rlen = Readmsg(pid, 2, rmbuf, sizeof(rmbuf));
	dumpbuf(rmbuf, rlen);
	puts("");

	printf("Receiver %d: write offset = %d, buf =      %s\n",getpid(), 5,lwmbuf);
	wlen = Writemsg(pid, 5, lwmbuf, strlen(lwmbuf));
	
	printf("Receiver %d:  read offset = %d, buf =   ",getpid(), 2);
	rlen = Readmsg(pid, 2, rmbuf, sizeof(rmbuf));
	dumpbuf(rmbuf, rlen);
	puts("");

	printf("Receiver %d:             reply buf = %s\n",getpid(), "****");
	Reply(pid,"****",4);
	sleep(1);
	puts("");

	return 0;
}

int main(int argc, char **argv)
{
	pid_t father, child, peer;
	char *name;
	
	if (argc < 3)
	{
		father = getpid();

		if ((child = fork()) == 0)
		{
			// Sender
			if (sender(father) == -1)
				perror("sender()");
		}
		else if (child != -1)
		{
			// Receiver
			if (receiver(0) == -1)
				perror("receiver()");
		}
		else
			perror("fork");

		exit(0);
	}

	name = argv[2];
	if (strcmp(argv[1],"-s") == 0)
	{
		// Sender
		if ((peer = qnx_name_locate(0,name,0,0)) == -1)
		{
			perror("qnx_name_locate");
			exit(1);
		}
		if (sender(peer) == -1)
			perror("sender()");
	}
	else if  (strcmp(argv[1],"-r") == 0)
	{
		// Receiver
		qnx_name_attach(0,name);
		if (receiver(0) == -1)
			perror("receiver()");
	}
	else
		printf("Usage: rwmsg [[-s | -r] name]\n");

	return 0;
}
