/*
 * tfork.c: tests the presence of the fork() bug
 *
 * procedure:
 *  parent registers
 *    forks
 *      child forks
 *        grandchild sends to grandparent
 *      child waits for grandchild
 *
 * $Id: tfork.c 132 2011-07-01 14:35:01Z meaton $
 */

#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <wait.h>

#include "srr.h"

int debug;

#define DEBUG !debug ? (void)0 : printf

int main(int argc, char* argv[])
{
	if(argc > 1) { debug = 1; }

	// parent registers
	if(SrrReg() == -1)
	{
		printf("SrrReg() failed: [%d] %s\n", errno, strerror(errno));

		exit(1);
	}

	DEBUG("parent: %d\n", getpid());

	switch(fork())
	{
		case -1:
			printf("fork() failed: [%d] %s\n", errno, strerror(errno));
			exit(1);

		case 0: {
			int parent = getppid();
			int child = fork();

			switch(child)
			{
				case -1:
					printf("fork() failed [%d] %s\n", errno, strerror(errno));
					exit(1);

				case 0:
					DEBUG("child %d sending to (grand-)parent %d\n", getpid(), parent);

					if(Send(parent, 0, 0, 0, 0) == -1)
					{
						DEBUG("Send() failed: [%d] %s\n", errno, strerror(errno));
						exit(errno);
					}
					assert(0); // unreachable

				default: {
					int status;
					int pid;

					DEBUG("middle %d forked child %d\n", getpid(), child);

					if((pid = waitpid(child, &status, 0)) == -1)
					{
						printf("waitpid() failed: [%d] %s\n", errno, strerror(errno));
						exit(1);
					}
					if(WIFEXITED(status) && WEXITSTATUS(status) == ESRCH)
					{
						printf("success: child %d exited with errno %d (ESRCH)!\n",
							pid, WEXITSTATUS(status));
						exit(0);
					}
					DEBUG("error: pid %d exited with status %#x\n", pid, status);
					exit(1);
				}
			} // end switch
		} // end middle
	}

	DEBUG("parent %d exiting\n", getpid());
	return 0;
}

