/*
 * srr_if.h: interface definitions for client library
 *
 *  Linux Loadable Module Implementation of QNX Send/Receive/Reply
 *	message passing, and related function.
 *
 *	QNX is a trademark of QSSL, Ltd. <www.qnx.com>
 *
 * Copyright (C) 2000 Cogent Real-Time Systems Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * $Id: srr_if.h 2544 2005-09-08 19:20:31Z andrew $
 */

#ifndef SRR_IF_H
#define SRR_IF_H

#include "srr.h"

/*
 * ioctl numeric codes
 */

/* The "type" identifying the s/r/r ioctls. */

#define	SRR_IOC_TYPE	'q'

#define	SRR_UNLOAD		0
#define	SRR_SEND		1
#define	SRR_RECV		2
#define	SRR_REPLY		3
#define	SRR_NAME_A		4
#define	SRR_NAME_L		5
#define	SRR_NAME_D		6
#define	SRR_PROXY_A		7
#define	SRR_PROXY_D		8
#define	SRR_TRIGGER		9
#define	SRR_TRACE		10
#define	SRR_PFLAGS		11
#define	SRR_TCREATE		12
#define	SRR_TDELETE		13
#define	SRR_TSET		14
#define	SRR_TGET		15
#define	SRR_KILL		16
/* 2000/02/17 - Alessandro Sala */
#define	SRR_READMSG	17
/* 2000/03/26 - Alessandro Sala */
#define	SRR_WRITEMSG	18
/* 2000/10/15 - John Green */
#define	SRR_SENDMX		19
#define	SRR_RECVMX		20
#define	SRR_REPLYMX		21
#define	SRR_READMSGMX	22
#define	SRR_WRITEMSGMX	23
#define	SRR_IRQ			24
#define	SRR_GETPID		25
#define	SRR_CGETRES		26
/* 2005/09/05 - Len Meakin */
#define SRR_RELAY		27
/* 2011/06/28 - Michael Eaton */
#define SRR_RECVTIME		28
#define SRR_SENDTIME    29
#define SRR_RECVMXTIME  30
#define SRR_SENDMXTIME  31
#define	SRR_IOC_MAXNR	31


	/* These are "write" ioctls, because the *struct* is written
	 * from the IPC client to the kernel.
	 */
#define SRR_IOCSEND		  _IOW (SRR_IOC_TYPE,SRR_SEND,    struct SrrSendArgs)
#define SRR_IOCRECV		  _IOW (SRR_IOC_TYPE,SRR_RECV,    struct SrrRecvArgs)
#define SRR_IOCREPLY	  _IOW (SRR_IOC_TYPE,SRR_REPLY,   struct SrrReplyArgs)
#define SRR_IOCNAME_A	  _IOW (SRR_IOC_TYPE,SRR_NAME_A,  struct SrrNameAttachArgs)
#define SRR_IOCNAME_D	  _IOW (SRR_IOC_TYPE,SRR_NAME_D,  struct SrrNameDetachArgs)
#define SRR_IOCNAME_L	  _IOW (SRR_IOC_TYPE,SRR_NAME_L,  struct SrrNameLocateArgs)
#define SRR_IOCPROXY_A	  _IOW (SRR_IOC_TYPE,SRR_PROXY_A, struct SrrProxyAttachArgs)
#define SRR_IOCPROXY_D	  _IOW (SRR_IOC_TYPE,SRR_PROXY_D, struct SrrProxyDetachArgs)
#define SRR_IOCTRIGGER	  _IOW (SRR_IOC_TYPE,SRR_TRIGGER, struct SrrTriggerArgs)
#define SRR_IOCUNLOAD	  _IO  (SRR_IOC_TYPE,SRR_UNLOAD)
#define SRR_IOCTRACE	  _IOW (SRR_IOC_TYPE,SRR_TRACE,   struct SrrTraceArgs)
#define SRR_IOCPFLAGS	  _IOWR(SRR_IOC_TYPE,SRR_PFLAGS,  struct SrrPFlagsArgs)
#define SRR_IOCTCREATE	  _IOW (SRR_IOC_TYPE,SRR_TCREATE, struct SrrTimerCreateArgs)
#define SRR_IOCTDELETE	  _IOW (SRR_IOC_TYPE,SRR_TDELETE, struct SrrTimerDeleteArgs)
#define SRR_IOCTSET		  _IOW (SRR_IOC_TYPE,SRR_TSET,    struct SrrTimerSetArgs)
#define SRR_IOCTGET		  _IOW (SRR_IOC_TYPE,SRR_TGET,    struct SrrTimerGetArgs)
#define SRR_IOCCGETRES	  _IOW (SRR_IOC_TYPE,SRR_CGETRES, struct SrrClockGetresArgs)
#define SRR_IOCKILL		  _IOW (SRR_IOC_TYPE,SRR_KILL,    struct SrrKillArgs)
/* 2000/02/17 - Alessandro Sala */
#define SRR_IOCREADMSG		_IOW (SRR_IOC_TYPE,SRR_READMSG, struct SrrReadmsgArgs)
/* 2000/03/27 - Alessandro Sala */
#define SRR_IOCWRITEMSG		_IOW (SRR_IOC_TYPE,SRR_WRITEMSG, struct SrrWritemsgArgs)

/* 2000/10/15 - John Green */
#define SRR_IOCSENDMX		  _IOW (SRR_IOC_TYPE,SRR_SENDMX, struct SrrSendmxArgs)
#define SRR_IOCRECVMX		  _IOW (SRR_IOC_TYPE,SRR_RECVMX, struct SrrRecvmxArgs)
#define SRR_IOCREPLYMX	  _IOW (SRR_IOC_TYPE,SRR_REPLYMX, struct SrrReplymxArgs)
#define SRR_IOCREADMSGMX		_IOW (SRR_IOC_TYPE,SRR_READMSGMX, struct SrrReadmsgmxArgs)
#define SRR_IOCWRITEMSGMX		_IOW (SRR_IOC_TYPE,SRR_WRITEMSGMX, struct SrrWritemsgmxArgs)
/* 2002/04/18 - Andrew Thomas */
#define SRR_IOCIRQ		_IOW (SRR_IOC_TYPE,SRR_IRQ, struct SrrIRQArgs)

#define SRR_IOCGETPID	_IOW (SRR_IOC_TYPE,SRR_GETPID, struct SrrGetpidArgs)

/* 2005/09/05 - Len Meakin */
#define SRR_IOCRELAY	  _IOW (SRR_IOC_TYPE,SRR_RELAY,   struct SrrRelayArgs)

/* 2011/06/28 - Michael Eaton */
#define SRR_IOCRECVTIME		_IOW (SRR_IOC_TYPE, SRR_RECVTIME, struct SrrRecvTimeoutArgs)
#define SRR_IOCSENDTIME _IOW (SRR_IOC_TYPE, SRR_SENDTIME, struct SrrSendTimeoutArgs)
#define SRR_IOCSENDMXTIME _IOW (SRR_IOC_TYPE, SRR_SENDMXTIME, struct SrrSendmxTimeoutArgs)
#define SRR_IOCRECVMXTIME _IOW (SRR_IOC_TYPE, SRR_RECVMXTIME, struct SrrRecvmxTimeoutArgs)
/*
 * ioctl interface structures
 */

typedef struct SrrTraceArgs
{
	int		level;
} SrrTrace;

	// send/receive/reply - I rely on pid's being first in these structs.
typedef struct SrrSendArgs
{
	pid_t	pid;
	size_t	ssize;
	size_t	rsize;
	void*	smsg;
	void*	rmsg;
} SrrSendArgs;

typedef struct SrrSendTimeoutArgs
{
	pid_t	pid;
	size_t	ssize;
	size_t	rsize;
	void*	smsg;
	void*	rmsg;
  unsigned long time;
} SrrSendTimeoutArgs;

typedef struct SrrRecvArgs
{
	pid_t	pid;
	int		nowait;
	size_t	size;
	void*	msg;
} SrrRecvArgs;

typedef struct SrrRecvTimeoutArgs
{
	pid_t	pid;
	int 	nowait;
	size_t	size;
	void*	msg;
	unsigned long time;
} SrrRecvTimeoutArgs;


typedef struct SrrReplyArgs
{
	pid_t	pid;
	size_t	size;
	void*	msg;
} SrrReplyArgs;

/* 2005/09/05 - Len Meakin */
typedef struct SrrRelayArgs
{
	pid_t	srcpid;
	pid_t	tgtpid;
} SrrRelayArgs;

/* 2000/02/17 - Alessandro Sala */
typedef struct SrrReadmsgArgs
{
	pid_t pid;
	size_t offset;
	size_t size;
	void* msg;
} SrrReadmsgArgs;

/* 2000/03/27 - Alessandro Sala */
typedef struct SrrWritemsgArgs
{
	pid_t pid;
	size_t offset;
	size_t size;
	void* msg;
} SrrWritemsgArgs;

/*
** Start MX
** 2000/10/15 - John Green
*/
typedef struct SrrSendmxArgs
{
        pid_t pid;
        size_t sparts;
        size_t rparts;
        void* smsgmx;
        void* rmsgmx;
} SrrSendmxArgs;

typedef struct SrrSendmxTimeoutArgs
{
        pid_t pid;
        size_t sparts;
        size_t rparts;
        void* smsgmx;
        void* rmsgmx;
        long time;
} SrrSendmxTimeoutArgs;

typedef struct SrrRecvmxArgs
{
        pid_t pid;
	int		nowait;
        size_t parts;
        void* msgmx;
} SrrRecvmxArgs;

typedef struct SrrRecvmxTimeoutArgs
{
        pid_t pid;
        int nowait;
        size_t parts;
        void* msgmx;
        long time;
} SrrRecvmxTimeoutArgs;

typedef struct SrrReplymxArgs
{
        pid_t pid;
        size_t parts;
        void* msgmx;
} SrrReplymxArgs;                 

typedef struct SrrReadmsgmxArgs
{
        pid_t pid;
        size_t offset;
        size_t parts;
        void* msgmx;
} SrrReadmsgmxArgs;
 
/* 2000/03/27 - Alessandro Sala */
typedef struct SrrWritemsgmxArgs
{
        pid_t pid;
        size_t offset;
        size_t parts;
        void* msgmx;
} SrrWritemsgmxArgs;        
/*
** End MX
*/

	// name attach/locate/detach
typedef struct SrrNameAttachArgs
{
	size_t	size;
	char*	name;
} SrrNameAttachArgs;

typedef struct SrrNameLocateArgs
{
	size_t	size;
	char*	name;
} SrrNameLocateArgs;

typedef struct SrrNameDetachArgs
{
	int		name_id;
} SrrNameDetachArgs;

	// proxy attach/detach/trigger
typedef struct SrrProxyAttachArgs
{
	pid_t	pid;
	char*	data;
	int		nbytes;
} SrrProxyAttachArgs;

typedef struct SrrProxyDetachArgs
{
	pid_t	pid;
} SrrProxyDetachArgs;

typedef struct SrrTriggerArgs
{
	pid_t	pid;
} SrrTriggerArgs;

	// process flags
typedef struct SrrPFlagsArgs
{
	pid_t	pid;
	unsigned short	flags;
	unsigned short	mask;
} SrrPFlagsArgs;

	// POSIX timer functions
typedef struct SrrTimerCreateArgs
{
	srr_clockid_t	clock;
	struct sigevent	event;
} SrrTimerCreateArgs;

typedef struct SrrTimerDeleteArgs
{
	srr_timer_t	tid;
} SrrTimerDeleteArgs;

typedef struct SrrTimerSetArgs
{
	srr_timer_t	tid;
	int		flags;
	const struct itimerspec* value;
	struct itimerspec* ovalue;
} SrrTimerSetArgs;

typedef struct SrrTimerGetArgs
{
	srr_timer_t	tid;
	struct itimerspec* value;
} SrrTimerGetArgs;

typedef struct SrrClockGetresArgs
{
	clockid_t tid;
	struct timespec* value;
} SrrClockGetresArgs;

typedef struct SrrKillArgs
{
	pid_t	pid;
	int		signo;
} SrrKillArgs;

#ifndef NO_IRQ
typedef struct SrrIRQArgs
{
	int8_t		type;		/* either SRRIRQ_IMMEDIATE or SRRIRQ_INTERRUPT */
	int8_t		irq_handle;	/* handle for this irq handler, from SBC_ATTACH */
	int16_t		len;		/* length of valid command in buffer */
	uint8_t		buffer[32];	/* the actual code */
	char		*extra;		/* extra payload pointer, if any. */
	uint32_t	extralen;	/* length of valid data in extra pointer */
} SrrIRQArgs;
#endif

typedef struct SrrGetpidArgs
{
	int32_t		dummy;		/* we need something in the structure? */
} SrrGetpidArgs;

#endif

